from .random.random import Random
from .idw.idw import IDW
from .spline.bspline import Spline
from .trend.trend import Trend
from .spatial.spatial import SpatialAverage
from .natural_neighbors.natural_neighbors import NaturalNeighbor
from .kriging.kriging import Kriging
from .gp.gp import GP
from .custom.custom import CustomInterpolator
