"""This python script contains all the constants that
might be needed in the various interpolation pacakages. 
"""

low_res = 10
med_res = 100
high_res = 1000

RESOLUTION = {"low": low_res, "standard": med_res, "high": high_res}
